#nullable disable
using Newtonsoft.Json;

namespace FitConnect.Models.v1.Api;

public partial class Problems {
    public enum ProblemInstanceEnum {
        Submission,
        Metadata,
        Data
    }

    public enum ProblemTypeEnum {
        AttachmentMismatch,
        MissingAttachments,
        EncryptionIssue,
        HashMismatch,
        IncorrectAuthenticationTag,
        InvalidEventLog,
        MissingAuthenticationTag,
        MissingData,
        MissingSchema,
        ServiceMismatch,
        SchemaViolation,
        SyntaxViolation,
        UnsupportedDataSchema,
        UnsupportedMetaSchema,
        UnsupportedService,
        UnsupportedReplyChannel
    }

    internal Problems() {
    }

    public Problems(ProblemTypeEnum problemType, string detail) {
        Type = "https://schema.fitko.de/fit-connect/events/problems/";
        switch (problemType) {
            case ProblemTypeEnum.MissingAttachments:
                Type += "missing-attachment";
                Title = TitleAttachmentsMismatch;
                Instance = "submission";
                break;
            case ProblemTypeEnum.AttachmentMismatch:
                Type += "attachments-mismatch";
                Title = TitleEventLogInconsistent;
                Instance = "submission";
                break;
            case ProblemTypeEnum.EncryptionIssue:
                Type += "encryption-issue";
                Title = TitleDecryptionIssue;
                Instance = "metadata";
                break;
            case ProblemTypeEnum.HashMismatch:
                Type += "hash-mismatch";
                Title = TitleEventLogInconsistent;
                Instance = "submission";
                break;
            case ProblemTypeEnum.IncorrectAuthenticationTag:
                Type += "incorrect-authentication-tag";
                Title = TitleEventLogInconsistent;
                Instance = "submission";
                break;
            case ProblemTypeEnum.InvalidEventLog:
                Type += "invalid-event-log";
                Title = TitleEventLogInconsistent;
                Instance = "submission";
                break;
            case ProblemTypeEnum.MissingAuthenticationTag:
                Type += "missing-authentication-tag";
                Title = TitleAuthenticationTagMissing;
                Instance = "submission";
                break;
            case ProblemTypeEnum.MissingData:
                Type += "missing-data";
                Title = TitleMissingData;
                Instance = "metadata";
                break;
            case ProblemTypeEnum.MissingSchema:
                Type += "missing-schema";
                Title = TitleMissingSchema;
                Instance = "metadata";
                break;
            case ProblemTypeEnum.ServiceMismatch:
                Type += "service-mismatch";
                Title = TitleServiceMismatch;
                Instance = "metadata";
                break;
            case ProblemTypeEnum.SchemaViolation:
                Type += "schema-violation";
                Title = TitleSchemaViolationData;
                Instance = "data";
                break;
            case ProblemTypeEnum.SyntaxViolation:
                Type += "syntax-violation";
                Title = TitleSyntaxViolation;
                Instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedDataSchema:
                Type += "unsupported-schema";
                Title = TitleUnsupportedDataSchema;
                Instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedMetaSchema:
                Type += "unsupported-schema";
                Title = TitleUnsupportedSchema;
                Instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedService:
                Type += "unsupported-service";
                Title = TitleUnsupportedService;
                Instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedReplyChannel:
                Type += "unsupported-reply-channel";
                Title = TitleUnsupportedReplyChannel;
                Instance = "metadata";
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(problemType), problemType, null);
        }

        this.Detail = detail;
    }


    public Problems(ProblemTypeEnum problemType, string title, string detail,
        string problemInstance) : this(problemType, detail) {
        this.Title = title ?? problemType switch {
            ProblemTypeEnum.InvalidEventLog => TitleEventLogInconsistent,
            _ => problemType.ToString()
        };

        Instance = problemInstance;
    }

    public Problems(ProblemTypeEnum problemType, string title, string detail,
        ProblemInstanceEnum problemInstance) : this(problemType, detail) {
        this.Title = title ?? problemType switch {
            ProblemTypeEnum.InvalidEventLog => TitleEventLogInconsistent,
            _ => problemType.ToString()
        };

        Instance = problemInstance switch {
            ProblemInstanceEnum.Metadata => "metadata",
            ProblemInstanceEnum.Submission => "submission",
            ProblemInstanceEnum.Data => "data",
            _ => throw new ArgumentOutOfRangeException(nameof(problemInstance), problemType, null)
        };
    }


    [JsonProperty("instance", NullValueHandling = NullValueHandling.Ignore)]
    public string Instance { get; set; }

    [JsonProperty("detail", NullValueHandling = NullValueHandling.Ignore)]
    public string Detail { get; set; }

    [JsonProperty("title", NullValueHandling = NullValueHandling.Ignore)]
    public string Title { get; set; }

    // [JsonProperty("description", NullValueHandling = NullValueHandling.Ignore)]
    // public string? description { get; set; }

    [JsonProperty("type", NullValueHandling = NullValueHandling.Ignore)]
    public string Type { get; set; }

    // [JsonProperty("items", NullValueHandling = NullValueHandling.Ignore)]
    // public Items? items { get; set; }
}
